@@ -34,6 +34,7 @@ class Transaction:
     def get_txid(self):
         """
         Get the transaction's ID.
+
         Returns:
             str: transaction ID
         """
@@ -46,8 +47,10 @@ class Transaction:
     def sign(self, private_key):
         """
         Sign the transaction with a private key.
+
         Args:
             private_key (str): the private key of the signing account
+
         Returns:
             SignedTransaction: signed transaction with the signature
         """
@@ -59,8 +62,10 @@ class Transaction:
     def raw_sign(self, private_key):
         """
         Sign the transaction.
+
         Args:
             private_key (str): the private key of the signing account
+
         Returns:
             bytes: signature
         """
@@ -110,19 +115,22 @@ class Transaction:
             "flat_fee": True,
             "lease": d["lx"] if "lx" in d else None,
         }
-        if d["type"] == constants.payment_txn:
+        txn_type = d["type"]
+        if not isinstance(d["type"], str):
+            txn_type = txn_type.decode()
+        if txn_type == constants.payment_txn:
             args.update(PaymentTxn._undictify(d))
             txn = PaymentTxn(**args)
-        elif d["type"] == constants.keyreg_txn:
+        elif txn_type == constants.keyreg_txn:
             args.update(KeyregTxn._undictify(d))
             txn = KeyregTxn(**args)
-        elif d["type"] == constants.assetconfig_txn:
+        elif txn_type == constants.assetconfig_txn:
             args.update(AssetConfigTxn._undictify(d))
             txn = AssetConfigTxn(**args)
-        elif d["type"] == constants.assetfreeze_txn:
+        elif txn_type == constants.assetfreeze_txn:
             args.update(AssetFreezeTxn._undictify(d))
             txn = AssetFreezeTxn(**args)
-        elif d["type"] == constants.assettransfer_txn:
+        elif txn_type == constants.assettransfer_txn:
             args.update(AssetTransferTxn._undictify(d))
             txn = AssetTransferTxn(**args)
         if "grp" in d:
@@ -149,6 +157,7 @@ class Transaction:
 class PaymentTxn(Transaction):
     """
     Represents a payment transaction.
+
     Args:
         sender (str): address of the sender
         fee (int): transaction fee (per byte if flat_fee is false)
@@ -165,6 +174,7 @@ class PaymentTxn(Transaction):
         lease (byte[32], optional): specifies a lease, and no other transaction
             with the same sender and lease can be confirmed in this
             transaction's valid rounds
+
     Attributes:
         sender (str)
         fee (int)
@@ -234,6 +244,7 @@ class PaymentTxn(Transaction):
 class KeyregTxn(Transaction):
     """
     Represents a key registration transaction.
+
     Args:
         sender (str): address of sender
         fee (int): transaction fee (per byte if flat_fee is false)
@@ -251,6 +262,7 @@ class KeyregTxn(Transaction):
         lease (byte[32], optional): specifies a lease, and no other transaction
             with the same sender and lease can be confirmed in this
             transaction's valid rounds
+
     Attributes:
         sender (str)
         fee (int)
@@ -326,15 +338,19 @@ class AssetConfigTxn(Transaction):
     """
     Represents a transaction for asset creation, reconfiguration, or
     destruction.
+
     To create an asset, include the following:
         total, default_frozen, unit_name, asset_name,
         manager, reserve, freeze, clawback, url, metadata,
         decimals
+
     To destroy an asset, include the following:
         index, strict_empty_address_check (set to False)
+
     To update asset configuration, include the following:
         index, manager, reserve, freeze, clawback,
         strict_empty_address_check (optional)
+
     Args:
         sender (str): address of the sender
         fee (int): transaction fee (per byte if flat_fee is false)
@@ -374,6 +390,7 @@ class AssetConfigTxn(Transaction):
             decimal. If set to 0, the asset is not divisible. If set to 1, the
             base unit of the asset is in tenths. Must be between 0 and 19,
             inclusive. Defaults to 0.
+
     Attributes:
         sender (str)
         fee (int)
@@ -554,6 +571,7 @@ class AssetFreezeTxn(Transaction):
     """
     Represents a transaction for freezing or unfreezing an account's asset
     holdings. Must be issued by the asset's freeze manager.
+
     Args:
         sender (str): address of the sender, who must be the asset's freeze
             manager
@@ -571,6 +589,7 @@ class AssetFreezeTxn(Transaction):
         lease (byte[32], optional): specifies a lease, and no other transaction
             with the same sender and lease can be confirmed in this
             transaction's valid rounds
+
     Attributes:
         sender (str)
         fee (int)
@@ -642,6 +661,7 @@ class AssetTransferTxn(Transaction):
     receiver, and set amount to 0.
     To revoke an asset, set revocation_target, and issue the transaction from
     the asset's revocation manager account.
+
     Args:
         sender (str): address of the sender
         fee (int): transaction fee (per byte if flat_fee is false)
@@ -662,6 +682,7 @@ class AssetTransferTxn(Transaction):
         lease (byte[32], optional): specifies a lease, and no other transaction
             with the same sender and lease can be confirmed in this
             transaction's valid rounds
+
     Attributes:
         sender (str)
         fee (int)
@@ -746,9 +767,11 @@ class AssetTransferTxn(Transaction):
 class SignedTransaction:
     """
     Represents a signed transaction.
+
     Args:
         transaction (Transaction): transaction that was signed
         signature (str): signature of a single address
+
     Attributes:
         transaction (Transaction)
         signature (str)
@@ -785,9 +808,11 @@ class SignedTransaction:
 class MultisigTransaction:
     """
     Represents a signed transaction.
+
     Args:
         transaction (Transaction): transaction that was signed
         multisig (Multisig): multisig account and signatures
+
     Attributes:
         transaction (Transaction)
         multisig (Multisig)
@@ -799,8 +824,10 @@ class MultisigTransaction:
     def sign(self, private_key):
         """
         Sign the multisig transaction.
+
         Args:
             private_key (str): private key of signing account
+
         Note:
             A new signature will replace the old if there is already a
             signature for the address. To sign another transaction, you can
@@ -844,11 +871,14 @@ class MultisigTransaction:
     def merge(part_stxs):
         """
         Merge partially signed multisig transactions.
+
         Args:
             part_stxs (MultisigTransaction[]): list of partially signed
                 multisig transactions
+
         Returns:
             MultisigTransaction: multisig transaction containing signatures
+
         Note:
             Only use this if you are given two partially signed multisig
             transactions. To append a signature to a multisig transaction, just
@@ -887,10 +917,12 @@ class MultisigTransaction:
 class Multisig:
     """
     Represents a multisig account and signatures.
+
     Args:
         version (int): currently, the version is 1
         threshold (int): how many signatures are necessary
         addresses (str[]): addresses in the multisig account
+
     Attributes:
         version (int)
         threshold (int)
@@ -1036,10 +1068,12 @@ class MultisigSubsig:
 
 class LogicSig:
     """
-    Represents a logic signature
+    Represents a logic signature.
+
     Arguments:
         logic (bytes): compiled program
         args (list[bytes]): args are not signed, but are checked by logic
+
     Attributes:
         logic (bytes)
         sig (bytes)
@@ -1078,8 +1112,10 @@ class LogicSig:
     def verify(self, public_key):
         """
         Verifies LogicSig against the transaction's sender address
+
         Args:
             public_key (bytes): sender address
+
         Returns:
             bool: true if the signature is valid (the sender address matches\
                 the logic hash or the signature is valid against the sender\
@@ -1113,6 +1149,7 @@ class LogicSig:
         """
         Compute hash of the logic sig program (that is the same as escrow
         account address) as string address
+
         Returns:
             str: program address
         """
@@ -1144,10 +1181,12 @@ class LogicSig:
     def sign(self, private_key, multisig=None):
         """
         Creates signature (if no pk provided) or multi signature
+
         Args:
             private_key (str): private key of signing account
             multisig (Multisig): optional multisig account without signatures
                 to sign with
+
         Raises:
             InvalidSecretKeyError: if no matching private key in multisig\
                 object
@@ -1164,8 +1203,10 @@ class LogicSig:
     def append_to_multisig(self, private_key):
         """
         Appends a signature to multi signature
+
         Args:
             private_key (str): private key of signing account
+
         Raises:
             InvalidSecretKeyError: if no matching private key in multisig\
                 object
@@ -1190,10 +1231,12 @@ class LogicSig:
 
 class LogicSigTransaction:
     """
-    Represents a logic signed transaction
+    Represents a logic signed transaction.
+
     Arguments:
         transaction (Transaction)
         lsig (LogicSig)
+
     Attributes:
         transaction (Transaction)
         lsig (LogicSig)
@@ -1206,6 +1249,7 @@ class LogicSigTransaction:
     def verify(self):
         """
         Verify LogicSig against the transaction
+
         Returns:
             bool: true if the signature is valid (the sender address matches\
                 the logic hash or the signature is valid against the sender\
@@ -1239,15 +1283,28 @@ class LogicSigTransaction:
                 self.transaction == other.transaction)
 
 
-def write_to_file(txns, path, overwrite=True):
+def write_to_file(objs, path, overwrite=True):
     """
     Write signed or unsigned transactions to a file.
+
     Args:
-        txns (Transaction[], SignedTransaction[], or MultisigTransaction[]):\
+        txns (Transaction[], SignedTransaction[], or MultisigTransaction[]):
             can be a mix of the three
         path (str): file to write to
         overwrite (bool): whether or not to overwrite what's already in the
             file; if False, transactions will be appended to the file
+
+    Returns:
+        bool: true if the transactions have been written to the file
+    """
+
+    """
+    Write objects to a file.
+    Args:
+        objs (Object[]): list of encodable objects
+        path (str): file to write to
+        overwrite (bool): whether or not to overwrite what's already in the
+            file; if False, transactions will be appended to the file
     Returns:
         bool: true if the transactions have been written to the file
     """
@@ -1258,41 +1315,34 @@ def write_to_file(txns, path, overwrite=True):
     else:
         f = open(path, "ab")
 
-    for txn in txns:
-        if isinstance(txn, Transaction):
-            enc = msgpack.packb({"txn": txn.dictify()}, use_bin_type=True)
-            f.write(enc)
+    for obj in objs:
+        if isinstance(obj, Transaction):
+            f.write(base64.b64decode(encoding.msgpack_encode({"txn": obj.dictify()})))
         else:
-            enc = msgpack.packb(txn.dictify(), use_bin_type=True)
-            f.write(enc)
+            f.write(base64.b64decode(encoding.msgpack_encode(obj)))
+
     f.close()
     return True
 
 
 def retrieve_from_file(path):
     """
-    Retrieve signed or unsigned transactions from a file.
+    Retrieve encoded objects from a file.
+
     Args:
         path (str): file to read from
+
     Returns:
-        Transaction[], SignedTransaction[], or MultisigTransaction[]:\
-            can be a mix of the three
+        Object[]: list of objects
     """
 
     f = open(path, "rb")
-    txns = []
+    objs = []
     unp = msgpack.Unpacker(f, raw=False)
-    for txn in unp:
-        if "msig" in txn:
-            txns.append(MultisigTransaction.undictify(txn))
-        elif "sig" in txn:
-            txns.append(SignedTransaction.undictify(txn))
-        elif "lsig" in txn:
-            txns.append(LogicSigTransaction.undictify(txn))
-        elif "type" in txn:
-            txns.append(Transaction.undictify(txn))
+    for obj in unp:
+        objs.append(encoding.msgpack_decode(obj))
     f.close()
-    return txns
+    return objs
 
 
 class TxGroup:
@@ -1322,8 +1372,10 @@ class TxGroup:
 def calculate_group_id(txns):
     """
     Calculate group id for a given list of unsigned transactions
+
     Args:
         txns (list): list of unsigned transactions
+
     Returns:
         bytes: checksum value representing the group id
     """
@@ -1345,11 +1397,13 @@ def calculate_group_id(txns):
 
 def assign_group_id(txns, address=None):
     """
-    Assign group id to a given list of unsigned transactions
+    Assign group id to a given list of unsigned transactions.
+
     Args:
         txns (list): list of unsigned transactions
         address (str): optional sender address specifying which transaction
-            return
+            to return
+
     Returns:
         txns (list): list of unsigned transactions with group property set
     """
